//-------------------------------------------------------------------
//
// op10_4_dato.cpp
//
// Klasse for hndtering av datoer. Datoer kan sammenliknes (like,
// fr, etter), og det er mulig  finne antall dager mellom to datoer.
//
// Flgende feilkontroll utfres i settfunksjonene:
//
// Aar >= 0. 
// 1 <= Mnd <= 12.
// Dag >= 1.
// For Mnd = 1,3,5,7,8,10,12 skal Dag <= 31.
// For Mnd = 4,6,9,11 skal Dag <= 30
// For Mnd = 2 skal Dag <= 28 hvis ikke skuddr, <= 29 ellers.
//       
#include <string>
#include <cctype>
#include <ctime>  // for  finne dagens dato

using namespace std;

class Dato {
  public:
    bool settDato(int nyDag,           // Inn
                  int nyMnd,           // Inn
                  int nyttAar);        // Inn                                                   
    bool settDato( 
           string nyDato); // Inn. Format: ddmmaaaa
    void settDato();  // dagens dato (ikke del av oppgaven)
           
    int  finnDag() const;
    int  finnMnd() const;
    int  finnAar() const;

    string   finnDato() const;                 
    int      sammenlikn( // Returverdi: 0 hvis likhet, <0 hvis 
                         // DenAndre etter, >0 hvis DenAndre fr den 
                         // datoen funksjonen kalles p vegne av.                           
               Dato denAndre) const; // Inn
    long int antallDagerTil(
               Dato denAndre) const; // Inn             
  private:      
    int  dag;
    int  mnd;
    int  aar;
  }; // Dato

//-------------------------------------------------------------------
//
// Implementasjon av klassen
//

using namespace std;

const int antDagerIAaret = 365;
const int antDagerISkuddaar = antDagerIAaret + 1;
const int datoLengde = 8;

// Lengden av mnedene er lagt inn som navnlse konstanter i
// funksjonen mndDager().

//-------------------------------------------------------------------
//
// Prototyper for hjelpefunksjoner. 
// Funksjonene er definert bakerst i denne filen.
//
bool datoOk(int dag, int mnd, int aar);
bool langMnd(int mnd);
int  mndDager(int mnd, int aar);                      
bool skuddaar(int aar);

//-------------------------------------------------------------------    
void Dato::settDato() {
  // Bruker funksjoner i <ctime> til  finne dagens dato
  time_t t;  // datatypen time_t representerer kalendertid
  time(&t);  
  tm *tid = localtime(&t);  // datatypen tm er en struktur 
  dag = tid->tm_mday;
  mnd = tid->tm_mon + 1;  // tm_mon gir verdi 0-11
  aar = tid->tm_year + 1900;  // tm_year gir r etter 1900
} // settDato
  
//-------------------------------------------------------------------    
bool Dato::settDato(int nyDag, int nyMnd, int nyttAar) {
  bool ok = true;
  if (!datoOk(nyDag, nyMnd, nyttAar)) {
    ok = false;    
  } else {
    dag = nyDag;
    mnd = nyMnd;
    aar = nyttAar;
  }
  return ok;
} // settDato (datoen gitt som tre heltall)

//-------------------------------------------------------------------                         
bool Dato::settDato(string nyDato) {
  bool ok = true;

  // Frst feilkontroll av forskjellig slag.
  
  // Lengden m vre (minst) lik datoLengde - 1.
  int lengde = nyDato.length();
  if (lengde < datoLengde - 1) ok = false;
  
  if (ok) {  
    // nyDato m bare inneholde sifre:
    int i = 0;
    while (i < lengde && isdigit(nyDato[i])) i++;
    if (i < lengde) ok = false;  // FEIL. Ikke-siffer er funnet

    if (ok) {     
      // Henter ut siffer 1 og 2 som dag, siffer 3 og 4 som mned og
      // siffer 5-8 som rstallet:
      int nyDag = (nyDato[0] - '0') * 10 + (nyDato[1] - '0');
      int nyMnd = (nyDato[2] - '0') * 10 + (nyDato[3] - '0');
      int nyttAar = (nyDato[4] - '0') * 1000 + (nyDato[5] - '0') * 100 +
              (nyDato[6] - '0') * 10 + (nyDato[7] - '0');    
  
      // sjekker om gyldige tall i datoen
      if (!datoOk(nyDag, nyMnd, nyttAar)) {
        ok = false;  // FEIL   
      } else {
        dag = nyDag;
        mnd = nyMnd;
        aar = nyttAar;
      }
    }
  }
  return ok;
} // settDato (datoen gitt som tekststreng)

//-------------------------------------------------------------------                         
int Dato::finnDag() const 
{ 
  return dag; 
} // finnDag

//-------------------------------------------------------------------                         
int Dato::finnMnd() const 
{ 
  return mnd; 
} // finnMnd

//-------------------------------------------------------------------                         
int Dato::finnAar() const 
{ 
  return aar; 
} // finnAar

//-------------------------------------------------------------------
string Dato::finnDato() const {
  // Lager en streng med riktig lengde og fyller den med blanke, kunne
  // ogs fylt den med et hvilket som helst annet tegn.
  string datoen(datoLengde, ' '); 
  datoen[0] = (char)(dag / 10 + '0');
  datoen[1] = (char)(dag % 10 + '0');
  datoen[2] = (char)(mnd / 10 + '0');
  datoen[3] = (char)(mnd % 10 + '0');
  datoen[4] = (char)(aar / 1000 + '0');
  int rest = aar % 1000;
  datoen[5] = (char)(rest / 100 + '0');
  rest %= 100;
  datoen[6] = (char)(rest / 10 + '0');
  datoen[7] = (char)(rest % 10 + '0');
  return datoen;
} // finnDato 

//-------------------------------------------------------------------
int Dato:: sammenlikn(Dato denAndre) const { 
  int like;                                               
  if (aar < denAndre.aar) like = -1;
  else if (aar > denAndre.aar) like = +1;
  else { // rstallene er like
    int mndDag1 = mndDager(mnd, aar) + dag;
    int mndDag2 = mndDager(denAndre.mnd, denAndre.aar) + denAndre.dag;
    if (mndDag1 < mndDag2) like = -1;
    else if (mndDag1 > mndDag2) like = +1;
    else like = 0;
    }
  return like;  
} // sammenlikn

//-------------------------------------------------------------------       
long int Dato::antallDagerTil(Dato denAndre) const {                       
  if (sammenlikn(denAndre) == 0) return 0;    // RETUR. Like datoer
  if (aar == denAndre.aar) { 
    return (mndDager(denAndre.mnd, denAndre.aar) + denAndre.dag -
            (mndDager(mnd, aar) + dag));      // RETUR. Samme r
    }        
            
  // Forskjellig r, ordner datoene i "riktig" rekkeflge
  Dato t1;
  Dato t2;
  
  if (sammenlikn(denAndre) > 0) {
    t1 = denAndre;  // datamedlemmene blir kopiert over
    t2.dag = dag;
    t2.mnd = mnd;
    t2.aar = aar;
    }
  else {
    t2 = denAndre; // datamedlemmene blir kopiert over
    t1.dag = dag;
    t1.mnd = mnd;
    t1.aar = aar;
    }                                                                   
    
  // antall hele r
  long int antDager = 0;
  int      aarNr;
  for (aarNr = t1.aar + 1; aarNr <= t2.aar - 1; aarNr++) {
    if (skuddaar(aarNr)) antDager += antDagerISkuddaar;
    else antDager += antDagerIAaret;
    }
  // antall dager i det frste ret
  if (skuddaar(t1.aar)) {
    antDager += antDagerISkuddaar - mndDager(t1.mnd, t1.aar) - t1.dag;
    }
  else {
    antDager += antDagerIAaret - mndDager(t1.mnd, t1.aar) - t1.dag;
    }                                                
  // antall dager i det siste ret
  antDager += mndDager(t2.mnd, t2.aar) + t2.dag;
                                 
  if (sammenlikn(denAndre) > 0) return -antDager;
  else return antDager;
} // AntalldagerTil                            
  
//-------------------------------------------------------------------
bool datoOk(int dag, int mnd, int aar) { // Retur true dersom OK
  bool ok = true;
  if (aar < 0) ok = false;            
  if (mnd < 1 || mnd > 12) ok = false;
  if (dag < 1) ok = false;              
  if (mnd == 2) {
   if (skuddaar(aar)) {
     if (dag > 29) ok = false;
     }
   else if (dag > 28) ok = false;
   }
  if (langMnd(mnd)) {
    if(dag > 31) ok = false; 
    }
  else {  // kort mned (februar allerede dekket)
    if (dag > 30) ok = false;  
    }
  return ok;
} // DatoOk

//-------------------------------------------------------------------
bool langMnd(int mnd) {   // Retur true hvis mnd har 31 dager
  if (mnd == 1 || mnd == 3 || mnd == 5 || mnd == 7 || mnd == 8 || 
      mnd == 10 || mnd == 12) return true;
  else return false;    
} // langMnd                    
                   
//-------------------------------------------------------------------
int mndDager(int mnd, int aar) {  // antall dager fram til 
                                  // (men ikke inkl.) mnd
  int antall = 0;
  if (mnd >  1) antall += 31;   // jan
  if (mnd >  2) {               // feb
    if (skuddaar(aar)) antall += 29;  
    else antall += 28;
    }
  if (mnd >  3) antall += 31;   // mars
  if (mnd >  4) antall += 30;   // april
  if (mnd >  5) antall += 31;   // mai
  if (mnd >  6) antall += 30;   // juni
  if (mnd >  7) antall += 31;   // juli
  if (mnd >  8) antall += 31;   // aug
  if (mnd >  9) antall += 30;   // sept
  if (mnd > 10) antall += 31;   // okt
  if (mnd > 11) antall += 30;   // nov
  return antall;
} // mndDager  
                          
//-------------------------------------------------------------------
bool skuddaar(int aar) {
  if (aar % 400 == 0 || aar % 4 == 0 && aar % 100 != 0) return true;
  else return false;
} // skuddaar 
                                           