//-------------------------------------------------------------------
//
// op11_2.cpp
//
// Kryptering. Programmet bruker den utvidete delen av ASCII-tabellen,
// Kontrollerer at de srnorske tegnene er korrekte, se funksjonene
// tilStor(), tilLiten() og storBokstav().    
//
// getline() brukes til  lese filen som skal krypteres/dekrypteres.
// For at siste linje p filen skal bli lest inn, m (br) den avsluttes
// med linjeskift.
//
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
using namespace std;

bool skalKryptere();                     
void aapneInnfil(int antForsok, ifstream &innfil);  // se kap. 11-1
void aapneUtfil(ofstream &utfil);
void aapneFiler(bool skalKrypteres, ifstream &inn, ofstream &ut);
void behandle(ifstream &inn, ofstream &ut, string fra, string til);
void oversett(string fra, string til, string &tekst);
char tilStor(char tegn);
char tilLiten(char tegn);
bool storBokstav(char tegn);

const int antForsok = 3;      // Nr brukeren skal skrive filnavn
int main()
{
  // Leser koder
  string   alfabet;
  string   koder;
  ifstream kodefil;
  
  cout << "Navn p kodefilen skal oppgis.\n";
  aapneInnfil(antForsok, kodefil);
  kodefil >> alfabet;
  kodefil >> koder;
  kodefil.close();

  if (alfabet.length() != koder.length()) {
    cout << "Lengde alfabet er " << alfabet.length() <<
            "\nLengde koder er " << koder.length() <<
            "\nDisse m vre like lange. Avbryter programutfrelsen.\n";
    exit(EXIT_FAILURE);
    }  
                       
  // Bestemmer hvilken vei krypteringen skal foreg
  bool skalKrypteres = skalKryptere();   // true dersom teksten skal krypteres
                                                   
  // pner filer
  ifstream inn;
  ofstream ut;                              
  aapneFiler(skalKrypteres, inn, ut);
                
  // Foretar krypteringen
  if (skalKrypteres) behandle(inn, ut, alfabet, koder);
  else behandle(inn, ut, koder, alfabet);

  inn.close();
  ut.close();
  
  return 0;
} // main     

//-------------------------------------------------------------------
//
// pner aktuelle datafiler. 
//
void aapneFiler(
  bool      skalKrypteres,     // Inn
  ifstream &inn,               // Ut
  ofstream &ut)                // Ut
{    
  if (skalKrypteres) {
    cout << "Innfilen er i klartekst." << endl;
    aapneInnfil(antForsok, inn);
    cout << "Utfilen blir kryptert." << endl;
    aapneUtfil(ut);
    } 
  else { 
    cout << "Innfilen er kodet." << endl;
    aapneInnfil(antForsok, inn);
    cout << "Utfilen blir i klartekst." << endl;
    aapneUtfil(ut);
    }                  
} // aapneFiler  

//-------------------------------------------------------------------
//
// Funksjonen henter all teksten fra innfil, konverterer den
// i henhold til tekststrengene til og fra, og sender resultatet
// til utfilen
//
void behandle(
  ifstream   &inn,    // Inn/ut 
  ofstream   &ut,     // Inn/ut
  string      fra,    // Inn
  string      til)    // Inn
{            
  string linje;     
  getline(inn, linje);     
  while (!inn.eof()) {                                                
    oversett(fra, til, linje);
    ut << linje << endl;
    getline(inn, linje);     
    }
  inn.clear();
} // behandle
                          
//-------------------------------------------------------------------
//
// Oversetter tekst i h.h.t. til og fra            
//
void oversett(
  string  fra,    // Inn. Det alfabet tekst er kodet i 
  string  til,    // Inn. Det alfabet tekst skal kodes til
  string &tekst)  // Inn/Ut.
{
  // Ytre for-lkke behandler alle tegnene i tekst
  int tegnNr;
  for (tegnNr = 0; tegnNr < tekst.length(); tegnNr++) {
                                                     
    bool stor = storBokstav(tekst[tegnNr]);
    tekst[tegnNr] = tilLiten(tekst[tegnNr]); // bare sm bokstaver i 
                                         // alfabetene
    // Sker etter tegnet i fra-alfabetet
    int i = 0;
    while (i < fra.length() && fra[i] != tekst[tegnNr]) i++;
    
    // Hvis tegnet er funnet foretas konvertering
    if (i < fra.length()) tekst[tegnNr] = til[i];
      
    // Konverterer eventuelt til stor bokstav
    if (stor) tekst[tegnNr] = tilStor(tekst[tegnNr]);
    } // ytre for-lkke
} // oversett                

//-------------------------------------------------------------------
//
// Dersom tegn er en liten bokstav, returneres den tilsvarende store
// bokstaven
//
char tilStor(
  char tegn) // Inn
{                                 
  if (tegn >= 'a' && tegn <= 'z') return tegn - ('a' - 'A');
  else if (tegn == '') return '';
  else if (tegn == '') return '';
  else if (tegn == '') return '';
  else return tegn;
} // tilStor           

//-------------------------------------------------------------------
//
// Dersom tegn er en stor bokstav, returneres den tilsvarende lille
// bokstaven
//
char tilLiten(
  char tegn) // Inn
{                                 
  if (tegn >= 'A' && tegn <= 'Z') return tegn + ('a' - 'A');
  else if (tegn == '') return '';
  else if (tegn == '') return '';
  else if (tegn == '') return '';
  else return tegn;
} // tilLiten          

//-------------------------------------------------------------------
//
// Returnerer true dersom tegn er en stor bokstav
//
bool storBokstav(
  char tegn)   // Inn
{
  return (bool)((tegn >= 'A' && tegn <= 'Z') ||
                 tegn == '' || tegn == '' || tegn == '');
} // storBokstav            

//-------------------------------------------------------------------
//
// Leser beskjed fra bruker om hvilken vei krypteringen skal g
// Returnerer true dersom kryptering skal foreg
//
bool skalKryptere()
{
  char svar;
  cout << "Skal du kryptere tekst, eller dekryptere ?" << endl;
  cout << "Skriv K for kryptering, D for dekryptering (D/K): ";
  cin >> svar;
  svar = tilStor(svar);
  while (svar != 'D' && svar != 'K') {
    cout << "Feil. Prv p nytt (D/K): ";
    cin >> svar;
    svar = tilStor(svar);
    } 
  return (svar == 'K');
} // skalKryptere  
                            
//-----------------------------------------------------
//
// Leser navnet p en utfil fra brukeren. pner filen.
// Dersom filen ikke eksisterer, blir den automatisk
// opprettet.
//
void aapneUtfil(
  ofstream   &utfil)          // Ut
{
  cout << "Oppgi filnavn, utfil: ";
  string filnavn;
  cin >> filnavn;
  utfil.clear();
  utfil.open(filnavn.c_str());
  if (!utfil) {
    cout << "Gir opp  pne filen. " << 
      "Avbryter programutfrelsen.\n";
    exit(EXIT_FAILURE);
    }  
} // aapneUtfil    

//-----------------------------------------------------
//
// Denne funksjonen er identisk med den foran,
// men den pner en innfil.
//
void aapneInnfil(
  int         antForsok,      // Inn
  ifstream   &innfil)          // Ut
{
  cout << "Oppgi filnavn, innfil: ";
  string filnavn;
  cin >> filnavn;
  innfil.clear();
  innfil.open(filnavn.c_str());
  int forsokNr = 1;
  while (!innfil && forsokNr < antForsok) {
    cout << "Fr ikke pnet filen. " << 
      "Oppgi filnavnet en gang til: ";  
    cin >> filnavn;
    innfil.clear(); 
    innfil.open(filnavn.c_str());
    forsokNr++;
    }                  
  if (!innfil) {
    cout << "Gir opp  pne filen. " << 
      "Avbryter programutfrelsen.\n";
    exit(EXIT_FAILURE);
    }  
} // aapneInnfil    

/*

Eksempler p datafiler:

Kodefilen kan se slik ut ("kode.txt"):

abcdefghijklmnopqrstuvwxyz
cdefghijklmnopqrstuvwxyzab

Filen i klartekst:

Det kom en bt med bananer ("test_inn.txt"). 
Og en bt til.

Den samme filen kryptert etter kodene foran:

Fgv mqo gp dbv ogf dcpcpgt. 
Qi gp dbv vkn.

*/
