//---------------------------------------------------------------------
//
// arbeidstaker.cpp
//
// Implementerer klassen Arbeidstaker.
//
//
#include <cmath>
#include <sstream>
#include <iomanip>
#include "arbeidstaker.h"

using namespace std;

//---------------------------------------------------------------------
Arbeidstaker::Arbeidstaker(const Navn &initNavn, const Dato &initFDato)
            : navn(initNavn), fdato(initFDato), ansDato(), tlf(0),
              loenn(0), prosent(0), arbtakernr(0)
{ } // Arbeidstaker

//---------------------------------------------------------------------
Arbeidstaker::Arbeidstaker(const Navn &initNavn, const Dato &initFDato,
                const Dato &initAnsDato, long int initTlf,
                long int initLoenn, double initSkPros,
                long int initArbTakNr)
            : navn(initNavn), fdato(initFDato), ansDato(initAnsDato),
              tlf(initTlf), loenn(initLoenn), prosent(initSkPros),
              arbtakernr(initArbTakNr)
{ } // Arbeidstaker

//---------------------------------------------------------------------
void Arbeidstaker::settAnsDato(const Dato &nyDato)
{
  ansDato = nyDato;
} // settAnsDato

//---------------------------------------------------------------------
void Arbeidstaker::settTlf(long int nyTlf)
{
  tlf = nyTlf;
} // settTlf

//---------------------------------------------------------------------
void Arbeidstaker::settLoenn(long int nyLoenn)
{
  loenn = nyLoenn;
} // settLoenn

//---------------------------------------------------------------------
bool Arbeidstaker::settSkPros(double nyPros)
{
  if (nyPros >= 0 && nyPros <= 100) {
    prosent = nyPros;
    return true;
    }
  else return false;
} // settSkPros

//---------------------------------------------------------------------
void Arbeidstaker::settArbTakNr(long int nyttNr)
{
  arbtakernr = nyttNr;
} // settArbTakNr

//---------------------------------------------------------------------
const Navn &Arbeidstaker::finnNavn() const
{
  return navn;
} // finnNavn

//---------------------------------------------------------------------
const Dato &Arbeidstaker::finnFDato() const
{
  return fdato;
} // finnFDato

//---------------------------------------------------------------------
const Dato &Arbeidstaker::finnADato() const
{
  return ansDato;
} // finnADato

//---------------------------------------------------------------------
long int Arbeidstaker::finnTlf() const
{
  return tlf;
} // finnTlf

//---------------------------------------------------------------------
long int Arbeidstaker::finnLoenn() const
{
  return loenn;
} // finnLoenn

//---------------------------------------------------------------------
double Arbeidstaker::finnSkPros() const
{
  return prosent;
} // finnSkPros

//---------------------------------------------------------------------
long int Arbeidstaker::finnArbTakNr() const
{
  return arbtakernr;
} // finnArbTakNr

//---------------------------------------------------------------------
int Arbeidstaker::finnAntallAarAnsatt() const
{
  Dato iDag;
  long int ansattDager = labs(ansDato.antallDagerTil(iDag));
  return (int) (ansattDager / 365L);
} // finnAntallAarAnsatt

//---------------------------------------------------------------------
bool Arbeidstaker::eldreEnn(const Arbeidstaker &denAndre) const
{
  return (fdato.sammenlikn(denAndre.fdato) < 0);
} // eldreEnn

//---------------------------------------------------------------------
bool Arbeidstaker::ansattLengerEnn(const Arbeidstaker &denAndre) const
{
  return (ansDato.sammenlikn(denAndre.ansDato) < 0);
} // ansattLengerEnn

//---------------------------------------------------------------------
string Arbeidstaker::lagUtskrift() const
{
  ostringstream utskrift;
  utskrift.setf(ios::fixed, ios::floatfield);
  utskrift << "Navn: " << navn.finnNavn() << ", fdt: " << fdato.finnDato();
  utskrift << ", ansatt: " << ansDato.finnDato() << endl;
  utskrift << "Tlf: " << tlf << ", lnn: " << loenn;
  utskrift << ", skatteprosent: " << setprecision(2) << prosent << endl;
  return utskrift.str();
} // lagUtskrift