//---------------------------------------------------------------------
//
// datafil.cpp
//
//
#include "arbeidstaker.h"
#include "datafil.h"
#include <iostream>
#include <sstream>
using namespace std;

Datafil::Datafil(const string &filnavn)
{
  fil.open(filnavn.c_str());
  if (!fil) {
    cout << "Feil ved pning av fil\n";
    exit(EXIT_FAILURE);
    }
} // Datafil

//---------------------------------------------------------------------
Datafil::~Datafil() {
  fil.clear();
  fil.close();
} // ~Datafil

//---------------------------------------------------------------------
Dato Datafil::finnFDatoEldst() 
{
  fil.clear();
  fil.seekg(0, ios::beg);
  Dato iDag;
  Navn ukjent("NN");
  Arbeidstaker eldst(ukjent, iDag);
  Arbeidstaker obj(ukjent, iDag);
  while (lesObjekt(obj)) {
    if (obj.eldreEnn(eldst)) eldst = obj;
    }
  return eldst.finnFDato();
} // finnFDatoEldst

//---------------------------------------------------------------------
Dato Datafil::finnAnsDatoAnsLengst() 
{
  fil.clear();
  fil.seekg(0, ios::beg);
  Dato iDag;
  Navn ukjent("NN");
  Arbeidstaker ansattLengst(ukjent, iDag);
  Arbeidstaker obj(ukjent, iDag);
  while (lesObjekt(obj)) {
    if (obj.ansattLengerEnn(ansattLengst)) ansattLengst = obj;
    }
  return ansattLengst.finnADato();
} // finnAnsDatoAnsLengst

//---------------------------------------------------------------------
string Datafil::lagUtskriftSammeFDato(const Dato &dato) 
{
  fil.clear();
  fil.seekg(0, ios::beg);
  ostringstream utskrift;
  Dato iDag;
  Navn ukjent("NN");
  Arbeidstaker obj(ukjent, iDag);
  while (lesObjekt(obj)) {
    if (obj.finnFDato().sammenlikn(dato) == 0) {
      utskrift << obj.lagUtskrift() << endl;
      }
    }
  return utskrift.str();
} // lagUtskriftSammeFDato

//---------------------------------------------------------------------
string Datafil::lagUtskriftSammeAnsDato(const Dato &dato) 
{
  fil.clear();
  fil.seekg(0, ios::beg);
  ostringstream utskrift;
  Dato iDag;
  Navn ukjent("NN");
  Arbeidstaker obj(ukjent, iDag);
  while (lesObjekt(obj)) {
    if (obj.finnADato().sammenlikn(dato) == 0) {
      utskrift << obj.lagUtskrift() << endl;
      }
    }
  return utskrift.str();
} // lagUtskriftSammeAnsDato

//---------------------------------------------------------------------
string Datafil::lagUtskriftAnsLengerEnnAar(int antallAar) 
{
  fil.clear();
  fil.seekg(0, ios::beg);
  ostringstream utskrift;
  Dato iDag;
  Navn ukjent("NN");
  Arbeidstaker obj(ukjent, iDag);
  while (lesObjekt(obj)) {
    if (obj.finnAntallAarAnsatt() >= antallAar) {
      utskrift << obj.lagUtskrift() << endl;
      }
    }
  return utskrift.str();
} // lagUtskriftAnsLengerEnnAar

//---------------------------------------------------------------------
double Datafil::finnSnittSkatteprosent() 
{
  fil.clear();
  fil.seekg(0, ios::beg);
  ostringstream utskrift;
  double sumPros = 0.0;
  int antall = 0;
  Dato iDag;
  Navn ukjent("NN");
  Arbeidstaker obj(ukjent, iDag);
  while (lesObjekt(obj)) {
    antall++;
    sumPros += obj.finnSkPros();
  }
  if (antall > 0) return (sumPros / (double) antall);
  else return -1;
} // finnSnittSkatteprosent

//---------------------------------------------------------------------
int Datafil::finnAntAnsatte() 
{
  fil.clear();
  fil.seekg(0, ios::beg);
  ostringstream utskrift;
  int antall = 0;
  Dato iDag;
  Navn ukjent("NN");
  Arbeidstaker obj(ukjent, iDag);
  while (lesObjekt(obj)) {
    antall++;
  }
  return antall;
} // finnAntAnsatte

//---------------------------------------------------------------------
bool Datafil::lesObjekt(Arbeidstaker &obj)    
{
  string   tekst;
  Navn     navn("");
  Dato     fdato;
  Dato     ansDato;
  long int tlf;
  long int loenn;
  double   prosent;
  long int arbtakernr;

  getline(fil, tekst);
  if (!fil.eof()) {  // mere data  lese
    navn.settNavn(tekst);
    getline(fil, tekst);
    fdato.settDato(tekst);
    getline(fil, tekst);
    ansDato.settDato(tekst);

    fil >> tlf;
    fil >> loenn;
    fil >> prosent;
    fil >> arbtakernr;
    fil.ignore(100, '\n');

    Arbeidstaker innlestObjekt(navn, fdato,ansDato, tlf, loenn, prosent, arbtakernr);
    obj = innlestObjekt;
    return true;
    }
  else return false;
} // lesObjekt