//-------------------------------------------------------------------
//
// penge.cpp - Implementasjon av klassen penger
//
#include <iostream>
#include "penge.h"
#include "..\nytte.h"

using namespace std;

const int    shillingPrPund  = 20;
const int    pencePrShilling = 12;
const int    pencePrPund = pencePrShilling * shillingPrPund;  
const string negativ = "Klassen penger hnterer ikke negative belp";
const string null = "Divisjon med null";

//-------------------------------------------------------------------
//
// Prototyper for funksjoner som benyttes ved implementasjon av 
// klassen penger. Funksjonene er definert bakerst i denne filen.
//
void   penceTilPsp(int pence, int &tilPund, int &tilShilling, 
                   int &tilPence);
int    pspTilPence(int pund, int shilling, int pence);

//-------------------------------------------------------------------          
void Penger::settAntPence(int pence)
{
  if (pence < 0) avsluttMedFeilmelding(negativ);
  antPence = pence;
} // settBelop

//-------------------------------------------------------------------          
void Penger::settBelop(int pund, int shilling = 0, int pence = 0)
{             
  if (pund < 0 || shilling < 0 || pence < 0) avsluttMedFeilmelding(
                                                    negativ);
  antPence = pspTilPence(pund, shilling, pence);
} // settBelop

//-------------------------------------------------------------------          
void Penger::finnBelop(int &pund, int &shilling, int &pence) const
{                  
  penceTilPsp(antPence, pund, shilling, pence);
} // finnBelop                   

//-------------------------------------------------------------------          
int Penger::finnBelop()
{             
  return antPence;
} // finnBelop

//-------------------------------------------------------------------          
Penger Penger::operator+(const Penger &denAndre) const
{
  Penger hjelp;
  hjelp = *this;
  hjelp += denAndre;
  return hjelp;
} // operator+

//-------------------------------------------------------------------          
Penger &Penger::operator+=(const Penger &denAndre) 
{
  antPence += denAndre.antPence;
  return *this;           
} // operator+=
  
//-------------------------------------------------------------------            
Penger Penger::operator-(const Penger &denAndre) const
{
  Penger hjelp;
  hjelp = *this;
  hjelp -= denAndre;
  return hjelp;
} // operator-

//-------------------------------------------------------------------          
Penger &Penger::operator-=(const Penger &denAndre) 
{
  antPence -= denAndre.antPence;
  return *this;           
} // operator-=

//-------------------------------------------------------------------          
Penger Penger::operator*(const Penger &denAndre) const
{
  Penger hjelp;
  hjelp = *this;
  hjelp *= denAndre;
  return hjelp;
} // operator*

//-------------------------------------------------------------------          
Penger &Penger::operator*=(const Penger &denAndre) 
{
  antPence *= denAndre.antPence;
  return *this;           
} // operator*=

//-------------------------------------------------------------------          
Penger Penger::operator/(const Penger &denAndre) const
{
  Penger hjelp;
  hjelp = *this;
  hjelp /= denAndre;
  return hjelp;
} // operator/

//-------------------------------------------------------------------          
Penger &Penger::operator/=(const Penger &denAndre) 
{
  if (denAndre.antPence == 0) avsluttMedFeilmelding(null);
  antPence /= denAndre.antPence;
  return *this;           
} // operator/=

//-------------------------------------------------------------------                    
bool Penger::operator<(const Penger &denAndre) const
{
  return sammenlikn(denAndre) < 0;
} // operator<

//-------------------------------------------------------------------          
bool Penger::operator>(const Penger &denAndre) const
{
  return sammenlikn(denAndre) > 0;
} // operator>

//-------------------------------------------------------------------
bool Penger::operator<=(const Penger &denAndre) const
{
  return sammenlikn(denAndre) <= 0;
} // operator<=

//-------------------------------------------------------------------
bool Penger::operator>=(const Penger &denAndre) const
{
  return sammenlikn(denAndre) >= 0;
} // operator>=

//-------------------------------------------------------------------
bool Penger::operator==(const Penger &denAndre) const
{
  return sammenlikn(denAndre) == 0;
} // operator==

//-------------------------------------------------------------------
bool Penger::operator!=(const Penger &denAndre) const
{
  return sammenlikn(denAndre) != 0;
} // operator!=

//-------------------------------------------------------------------
Penger Penger::operator/(int heltall) const
{                                                       
  if (heltall == 0) avsluttMedFeilmelding(null);
  else if (heltall < 0) avsluttMedFeilmelding(negativ);
  Penger hjelp;
  hjelp.settAntPence(antPence / heltall);
  return hjelp;
} // operator/

//-------------------------------------------------------------------
Penger Penger::operator*(int heltall) const
{                                                      
  if (heltall < 0) avsluttMedFeilmelding(negativ);
  Penger hjelp;
  hjelp.settAntPence(antPence * heltall);
  return hjelp;
} // operator*

    
//-------------------------------------------------------------------            
bool Penger::operator>(int heltall) const
{
  return antPence > heltall;
} // operator>

//-------------------------------------------------------------------            
bool Penger::operator<(int heltall) const
{
  return antPence < heltall;
} // operator<

//-------------------------------------------------------------------          
int Penger::sammenlikn(const Penger &denAndre) const
{
  if (antPence > denAndre.antPence) return 1;
  else if (antPence < denAndre.antPence) return -1;
  else return 0;
} // sammenlikn

//-------------------------------------------------------------------
//
// Funksjon som regner om fra pence 
// til et antall pund, shilling og pence
//
void penceTilPsp(
  int  pence,                 // Inn
  int &tilPund,               // Ut
  int &tilShilling,           // Ut
  int &tilPence)              // Ut
{
  tilPund = pence / pencePrPund;
  pence %= pencePrPund;
  tilShilling = pence / pencePrShilling;
  tilPence = pence % pencePrShilling;
} // penceTilPsp  

//-------------------------------------------------------------------
//
// Funksjon som regner om fra pund, shilling og pence 
// til et antall pence
//
int pspTilPence(
  int  pund,                  // Inn
  int  shilling,              // Inn
  int  pence)                 // Inn
{
  return pund * pencePrPund + shilling * pencePrShilling + pence;
} // pspTilPence
          
