//-----------------------------------------------------------------------------
//
// Tall.cpp - implementasjonsfil
//                                                                 
// Alle operasjonene forutsetter at tallene er ok, det vil si ikke-negative.
//

#include <iostream>
#include <string> 
#include <cctype>              
#include "tall.h"
 
//-----------------------------------------------------------------------------
Tall::Tall()
{ } // Tall

//-----------------------------------------------------------------------------
Tall::Tall(int initTall)
{
  if (initTall > 0) {
    int nr = 0;
    while (initTall > 0) {
      sifrene.insert(sifrene.begin(), initTall % 10);
      initTall /= 10;
      nr++;
      }
    }
  else sifrene.push_back(0);  // bare sifferet 0
} // Tall
        
//-----------------------------------------------------------------------------
Tall::Tall(const string &initTall)
{
  string tekst = initTall;
  int antSiffer = (int) tekst.length();
  for (int nr = 0; nr < antSiffer; nr++) {
    sifrene.push_back(tekst[nr] - '0'); 
    }
} // Tall                                                 

//-----------------------------------------------------------------------------
string Tall::finnTall() const
{
  int antSiffer = (int) sifrene.size();
  string tallet(antSiffer, ' ');
  for (int nr = 0; nr < antSiffer; nr++) {
    tallet[nr] = sifrene[nr] + '0';
    }
  return tallet;
} // finnTall      

//-----------------------------------------------------------------------------
Tall Tall::operator+(const Tall &detAndre) const
{
  Tall hjelp = *this;
  hjelp += detAndre;
  return hjelp;
} // operator+

//-----------------------------------------------------------------------------
Tall Tall::operator-(const Tall &detAndre) const
{
  Tall hjelp = *this;
  hjelp -= detAndre;
  return hjelp;
} // operator-

//-----------------------------------------------------------------------------
Tall &Tall::operator+=(const Tall &detAndre)
{        
  Tall addend = detAndre; // kopi som vi kan endre p
  
  int antSiffer1 = (int) sifrene.size();
  int antSiffer2 = (int) addend.sifrene.size();
  
  /*
  cout << "AAAA";
  sifrene.insert(sifrene.begin(), 0);   
  cout << "BBB"  << finnTall() << endl;;
  
  addend.sifrene.insert(addend.sifrene.begin(), 0);   
  cout << "CCCC" << addend.finnTall() << endl;
  */
  // Setter nuller forrest i det tallet som har frrest sifre
  int nr;
  for (nr = antSiffer1; nr < antSiffer2; nr++) {
    sifrene.insert(sifrene.begin(), 0);   
    }

  for (nr = antSiffer2; nr < antSiffer1; nr++) {
    addend.sifrene.insert(addend.sifrene.begin(), 0);   
    }
  
  // N har tallene like mange sifre, og vi kan legge sammen
  int antSiffer = (antSiffer1 > antSiffer2) ? antSiffer1 : antSiffer2;
  int mente = 0;   
  int tall;
  for (nr = antSiffer - 1; nr >= 0; nr--) {
    tall = sifrene[nr] + addend.sifrene[nr] + mente;
    sifrene[nr] = tall % 10;
    mente = tall / 10; 
    }   
  if (mente > 0) sifrene.insert(sifrene.begin(), mente);
  return *this;
} // operator+=    
        
//-----------------------------------------------------------------------------
Tall &Tall::operator-=(const Tall &detAndre)
{ 
  if (sifrene == detAndre.sifrene) { // tallene er like
    sifrene.clear();
    sifrene.push_back(0);
    }
  else {  // *this er strre enn detAndre, vanlig subtraksjon

    Tall subtrahend = detAndre; // kopi som vi kan endre p
    int antSiffer1 = (int) sifrene.size();
    int antSiffer2 = (int) subtrahend.sifrene.size();
    
    // Setter nuller frst i subtrahenden, da blir det enklere  trekke fra
    int nr;
    for (nr = antSiffer2; nr < antSiffer1; nr++) {
      subtrahend.sifrene.insert(subtrahend.sifrene.begin(), 0);   
      }

    // N har tallene like mange sifre, og vi kan trekke fra
    int tall;
    for (nr = antSiffer1 - 1; nr >= 0; nr--) {
      tall = sifrene[nr] - subtrahend.sifrene[nr];
      
      if (tall < 0) { // vi m lne
        tall += 10;
        sifrene[nr - 1]--;  // her lner vi fra
        }
      sifrene[nr] = tall;
      } // for
    
    // fjerner nuller i begynnelsen av resultatet
    while (sifrene[0] == 0) {
      sifrene.erase(sifrene.begin());
      } // while
    } // else
  return *this;
} // operator-=

//-----------------------------------------------------------------------------
bool Tall::operator<(const Tall &detAndre) const
{
  return (sammenlikn(detAndre) < 0);
} // operator<             

//-----------------------------------------------------------------------------
bool Tall::operator<=(const Tall &detAndre) const
{
  return (sammenlikn(detAndre) <= 0);
} // operator<=  

//-----------------------------------------------------------------------------
Tall &Tall::operator++(void)
{
  *this += 1;
  return *this;
} //operator++      

//-----------------------------------------------------------------------------
int Tall::sammenlikn(const Tall &detAndre) const
{
  int antSiffer1 = (int) sifrene.size();
  int antSiffer2 = (int) detAndre.sifrene.size();
  if (antSiffer1 < antSiffer2) return -1;
  else if (antSiffer1 > antSiffer2) return 1;
  else { // tallene er like lange      
    if (sifrene == detAndre.sifrene) return 0; // tallene er like

    // finner frste siffer som er forskjellig
    int nr = 0;
    while (nr < antSiffer1 && sifrene[nr] == detAndre.sifrene[nr]) nr++;
    return sifrene[nr] - detAndre.sifrene[nr];
    }
} // sammenlikn

//-----------------------------------------------------------------------------
ostream &operator<<(ostream &ut, const Tall &tall)
{             
  ut << tall.finnTall();
  return ut;
} // operator<<

//-----------------------------------------------------------------------------
istream &operator>>(istream &inn, Tall &tall)
{ 
  string tallSomTekst;
  char tegn = inn.get();
  while (isdigit(tegn)) {
    tallSomTekst.append(1, tegn);
    tegn = inn.get();
    }
  tall = Tall(tallSomTekst);
  return inn;
} // operator>>  
