//-------------------------------------------------------------------
//
// moenster.cpp - implementasjon av mnsterklassene
//
#include <fstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
#include <algorithm>
#include <cstdlib>
#include "moenster.h"

using namespace std;

void avsluttProgrammet(const string &melding); // hjelpefunksjon

//-------------------------------------------------------------------
//
//  Klassen Moenster
//
Moenster::Moenster()
{  
} // Moenster

//-------------------------------------------------------------------
Moenster::Moenster(const farge initTabell[hoyde][bredde])
{
  settMoenster(initTabell);
} // Moenster

//-------------------------------------------------------------------
void Moenster::settMoenster(const farge nyttMoenster[hoyde][bredde])
{
  for (int i = 0; i < hoyde; i++) {
    for (int j = 0; j < bredde; j++) tabell[i][j] = nyttMoenster[i][j];
    }
} // settMoenster

//-------------------------------------------------------------------
void  Moenster::finnMoenster(farge utTabell[hoyde][bredde]) const
{
  for (int i = 0; i < hoyde; i++) {
    for (int j = 0; j < bredde; j++) utTabell[i][j] = tabell[i][j];
    }
} // finnMoenster

//-------------------------------------------------------------------
int Moenster::finnAntRuter(farge fargen) const 
{
  int antRuterMedDenneFarge = 0;  
  for (int i = 0; i < hoyde; i++) {
    for (int j = 0; j < bredde; j++) {
      if (tabell[i][j] == fargen) antRuterMedDenneFarge++;
      } // for
    }
  return antRuterMedDenneFarge;
} // finnAntRuter

//-------------------------------------------------------------------
bool Moenster::operator==(const Moenster &detAndre) const
{
  return (sammenlikn(detAndre) == 0);
} // operator==

//-------------------------------------------------------------------
bool Moenster::operator!=(const Moenster &detAndre) const
{
  return (sammenlikn(detAndre) != 0);
} // operator!=

//-------------------------------------------------------------------
bool Moenster::operator>(const Moenster &detAndre) const
{
  return (sammenlikn(detAndre) > 0);
} // operator>

//-------------------------------------------------------------------

bool Moenster::operator>=(const Moenster &detAndre) const
{
  return (sammenlikn(detAndre) >= 0);
} // operator>=

//-------------------------------------------------------------------
bool Moenster::operator<(const Moenster &detAndre) const
{
  return (sammenlikn(detAndre) < 0);
} // operator>

//-------------------------------------------------------------------

bool Moenster::operator<=(const Moenster &detAndre) const
{
  return (sammenlikn(detAndre) <= 0);
} // operator>=

//-------------------------------------------------------------------
int Moenster::sammenlikn(const Moenster &detAndre) const
{
  int res = finnAntRuter(roed) - detAndre.finnAntRuter(roed);
  if (res == 0) res = finnAntRuter(svart) - detAndre.finnAntRuter(svart);
  return res;
} // sammenlikn

//-------------------------------------------------------------------
ostream &operator<<(ostream &ut, const Moenster &moenster)
{
  Moenster::farge tabell[Moenster::hoyde][Moenster::bredde];
  moenster.finnMoenster(tabell);
  for (int rad = 0; rad < Moenster::hoyde; rad++) {
    for (int kolonne = 0; kolonne < Moenster::bredde; kolonne++) {
      string tekst;
      int tallverdi = tabell[rad][kolonne];
      if (tallverdi == Moenster::hvit) tekst = "hvit ";
      else if (tallverdi == Moenster::roed) tekst = "rd  ";
      else tekst = "svart";
      ut << tekst << " ";
      } // for kolonne
    ut << endl;
    } // for rad
  ut << endl;
  return ut;
} // operator<<

//-------------------------------------------------------------------
bool lesFraBinFil(fstream &datafil, Moenster &moenster)
{
  Moenster::farge tabell[Moenster::hoyde][Moenster::bredde];
  datafil.read((char *)tabell,       
      sizeof(Moenster::farge) * Moenster::hoyde * Moenster::bredde);
  if (datafil.bad()) {
    avsluttProgrammet(
      "Problemer med datafilen i funksjonen lesFraBinFil()");
  }
  if (!datafil.eof()) {
    moenster.settMoenster(tabell);
    return true;
    }
  else return false;
} // lesFraBinFil

//-------------------------------------------------------------------
void skrivTilBinFil(fstream &datafil, const Moenster &moenster)
{
  Moenster::farge tabell[Moenster::hoyde][Moenster::bredde];
  moenster.finnMoenster(tabell);
  datafil.write((char *)tabell,       
      sizeof(Moenster::farge) * Moenster::hoyde * Moenster::bredde);
  if (datafil.bad()) {
    avsluttProgrammet(
      "Problemer med datafilen i funksjonen skrivTilBinFil()");
  }
} // skrivTilBinFil

//-------------------------------------------------------------------
//
//  Klassen Moenstersamling
//

Moenstersamling::Moenstersamling(const string &filnavn)
{
  datafil.open(filnavn.c_str(), ios::in | ios::out | ios::binary); 
  if (datafil.bad()) avsluttProgrammet("Feil ved pning av datafil.\n");

  Moenster nyttMoenster;  
  while (lesFraBinFil(datafil, nyttMoenster)) {
    samling.push_back(nyttMoenster);
    }
  datafil.clear(); // eof() har satt feilmerke p filen
  sortert = false;
} // Moenstersamling

//-------------------------------------------------------------------
Moenstersamling::~Moenstersamling()
{
  datafil.clear();
  datafil.seekp(ios::beg);
  if (datafil.bad()) {
    avsluttProgrammet("Problemer med datafilen i destruktren.");
    }
  for (int i = 0; i < (int) samling.size(); i++) {
    skrivTilBinFil(datafil, samling[i]);
    }
  datafil.close();
} // ~Moenstersamling

//-------------------------------------------------------------------
bool Moenstersamling::settInnNyttMoenster(const Moenster &nyttMoenster)
{
  if (!binary_search(samling.begin(), samling.end(), nyttMoenster)) {
    samling.push_back(nyttMoenster);
    return true;
    }
  else return false;
} // settInnNyttMoenster

//-------------------------------------------------------------------
int Moenstersamling::finnAntMoenstre() const
{
  return (int) samling.size();
} // finnAntMoenstre

//-------------------------------------------------------------------
const Moenster *Moenstersamling::finnMoenster(int indeks) const
{
  if (indeks >= 0 && indeks < (int) samling.size()) {
    return &samling[indeks];
    }
  else return 0;
} // finnMoenster

//-------------------------------------------------------------------
int Moenstersamling::finnMoenster(const Moenster &moensteret) const 
{
  vector<Moenster>::const_iterator pos = 
    lower_bound(samling.begin(), samling.end(), moensteret);

  if (*pos == moensteret) { // mnsteret er funnet
    return ((int) (pos - samling.begin()));
    }
  else return -((int)(pos - samling.begin()));
} // finnMoenster

//-------------------------------------------------------------------
void Moenstersamling::sorter()
{
  if (!sortert) {
    sort(samling.begin(), samling.end());
    sortert = true;
  }
} // sorter

//-------------------------------------------------------------------
//
// Funksjonen brukes ved alvorlige feil, her er det snakk om problemer
// med datafilen
//
void avsluttProgrammet(
       const string &melding)    // Inn
{
  cerr << "Programmet avsluttes p grunn av alvorlig feil: "; 
  cerr << melding << endl;
  exit(EXIT_FAILURE);
}  // avsluttProgrammet