//---------------------------------------------------------------------
//
// ansatte.h
//
// Klasser for hndtering av ulike kategorier ansatte
//
#ifndef ANSATTE_DEFINERT
  #define ANSATTE_DEFINERT

  #include <string>
  using namespace std;

  class Ansatt {
    public:
      const string  &finnNavn() const;
      const string  &finnFDato() const;
      virtual char   finnArbAvtale() const = 0;
      virtual string finnKlasse() const;

    protected:
      Ansatt(
        const string &initNavn,         // Inn
        const string &initFDato);       // Inn

    private:
      string navn;
      string fdato;
  };  // Ansatt

  //---------------------------------------------------------------------
  class TeknPers : public Ansatt {
    public:
      TeknPers(
        const string &initNavn,         // Inn
        const string &initFDato,        // Inn
        const string &initSpesOmr);     // Inn

      const string  &finnSpesOmr() const;
      virtual char   finnArbAvtale() const;
      virtual string finnKlasse() const;

    private:
      string spesialOmraade;
  }; // TeknPers

  //---------------------------------------------------------------------
  class Laerer : public Ansatt {
    public:
      Laerer(
        const string &initNavn,         // Inn
        const string &initFDato,        // Inn
        const string &initFagkrets);    // Inn

      const string  &finnFagkrets() const;
      virtual char   finnArbAvtale() const;
      virtual string finnKlasse() const;

    private:
      string fagkrets;
  }; // Laerer

  //---------------------------------------------------------------------
  class AdmAns : public Ansatt {
    public:
      virtual char   finnArbAvtale() const;
      virtual string finnKlasse() const;

    protected:
      AdmAns(
        const string &initNavn,         // Inn
        const string &initFDato);       // Inn
  }; // AdmAns

  //---------------------------------------------------------------------
  class Sekretaer : public AdmAns {
    public:
      Sekretaer(
        const string &initNavn,         // Inn
        const string &initFDato,        // Inn
        const string &initSystem);      // Inn

      const string  &finnSystem() const;
      virtual string finnKlasse() const;

    private:
      string system;
  }; // Sekretaer

  //---------------------------------------------------------------------
  class AndreAdmAns : public AdmAns {
    public:
      AndreAdmAns(
        const string &initNavn,         // Inn
        const string &initFDato);       // Inn

      virtual string finnKlasse() const;

  }; // AndreAdmAns
#endif