//-------------------------------------------------------------------
//
// bibliotek.h - en samling med bker
//

#ifndef BIBLIOTEK_DEFINERT
  #define BIBLIOTEK_DEFINERT

  #include <string>
  #include <vector>
  #include "boeker.h"

  using namespace std;

  class Bibliotek {
    public:
      Bibliotek();
      ~Bibliotek();

      //
      // Funksjonen legger peker til bok-objektet inn i vektoren.
      // Objektet m ligge i det frie lageret. Destruktren
      // srger for  slette objektene.
      //
      // Dersom bok med det samme ISBN fins fra fr, registreres
      // ikke boka. Da returnerer funksjonen false.
      //
      bool       registrerNyBok(
                   const Bok *&nyBok);          // Inn/ut (pekeren settes lik 0)

      // Flgende to funksjoner kan brukes til  hente ut alle bkene
      int        finnAntallBoeker() const;
      const Bok *finnBok(  // returnerer 0 hvis ugyldig indeks
                   int indeks) const;           // Inn

      // Denne funksjonen returnerer en vektor med pekere til
      // alle bker med den gitte tittelen. Vektoren er tom
      // hvis ingen bker med denne tittelen fins.
      vector<const Bok *> finnBoeker(  // returnerer 0 hvis tittel ikke finnes
                          const string &tittel) const; // Inn

    private:
      // Klassen inneholder en vektor av pekere med objektene i det frie lageret.
      // Det tilsier at kopikonstruktr, tilordningsoperator og destruktr
      // m programmeres - dersom det er behov for dem. Den siste
      // m vi lage, de to frste anser vi som undvendige.
      // Derfor deklarerer vi dem privat, da vil klienter ikke kunne
      // n dem.
      Bibliotek(
        const Bibliotek &original);  // Inn
      Bibliotek &operator=(
        const Bibliotek& denAndre);  // Inn

      // Vektor av const-pekere, dvs at info om bkene ikke kan forandres
      vector<const Bok *> boeker;
  }; // Bibliotek
#endif