//---------------------------------------------------------------------
//
// boeker.cpp - implementasjonsfil
//

#include <sstream>
#include "boeker.h"

using namespace std;

Bok :: Bok(const string &initTittel, const string *initForft,
           int initAntForft, int initUtgAar, double initPris,
           int initAntSider, const string &initIsbn,
           const string &initUtgLand)
     : tittel(initTittel),
       utgAar(initUtgAar),
       pris(initPris),
       antSider(initAntSider),
       isbn(initIsbn),
       utgLand(initUtgLand)
{
  forfattere = 0;
  antForft = 0;
  kopierForftTabell(initForft, initAntForft);
} // Bok

//---------------------------------------------------------------------
Bok::Bok(const Bok& original)
   : tittel(original.tittel),
     utgAar(original.utgAar),
     pris(original.pris),
     antSider(original.antSider),
     isbn(original.isbn),
     utgLand(original.utgLand)
{
  kopierForftTabell(original.forfattere, original.antForft);
} // Bok

//---------------------------------------------------------------------
Bok &Bok::operator=(const Bok& denAndre)
{
  if (this != &denAndre) {
    tittel = denAndre.tittel;
    utgAar = denAndre.utgAar;
    pris = denAndre.pris;
    antSider = denAndre.antSider;
    isbn = denAndre.isbn;
    utgLand = denAndre.utgLand;
    kopierForftTabell(denAndre.forfattere, denAndre.antForft);
  }
  return *this;
} // operator=

//---------------------------------------------------------------------
Bok::~Bok()
{
  if (forfattere != 0) delete [] forfattere;
} // ~Bok

//---------------------------------------------------------------------
void Bok::kopierForftTabell(const string *initForft, int initAntForft)
{
  if (forfattere != 0) delete [] forfattere;
  antForft = initAntForft;
  forfattere = new string[antForft];
  for (int i = 0; i < antForft; i++) forfattere[i] = initForft[i];
} // kopierForftTabell

//---------------------------------------------------------------------
const string &Bok::finnTittel() const
{
  return tittel;
} // finnTittel

//---------------------------------------------------------------------
const string *Bok::finnForfattere() const
{
  return forfattere;
} // finnForfattere

//---------------------------------------------------------------------
int Bok::finnAntForft() const
{
  return antForft;
} // finnAntForft

//---------------------------------------------------------------------
int Bok::finnUtgAar() const
{
  return utgAar;
} // finnUtgAar

//---------------------------------------------------------------------
double Bok::finnPris() const
{
  return pris;
} // finnPris

//---------------------------------------------------------------------
int Bok::finnAntSider() const
{
  return antSider;
} // finnAntSider

//---------------------------------------------------------------------
const string &Bok::finnIsbn() const
{
  return isbn;
} // finnIsbn

//---------------------------------------------------------------------
const string &Bok::finnUtgLand() const
{
  return utgLand;
} // finnUtgLand

//---------------------------------------------------------------------
string Bok::lagUtskrift() const
{
  ostringstream res;
  res << "Type bok: " << finnBoktype() << endl;
  res << "Forfatter(e): ";
  for (int i = 0; i < antForft - 1; i++) res << forfattere[i] << ", ";
  res << forfattere[antForft - 1] << endl;

  res << "Tittel: " << tittel << "\nUtgitt: " << utgAar << " i "
      << utgLand << ", isbn: " << isbn << ", ant sider: " << antSider
      << ", pris: " << pris << "kr.";
  return res.str();
}; // lagUtskrift

//-------------------------------------------------------------------
Barnebok::Barnebok(const string &initTittel, const string *initForft,
           int initAntForft, int initUtgAar, double initPris,
           int initAntSider, const string &initIsbn,
           const string &initUtgLand, int initFraAar, int initTilAar)
        : Bok(initTittel, initForft, initAntForft, initUtgAar, initPris, 
              initAntSider, initIsbn, initUtgLand),
          fraAar(initFraAar),
          tilAar(initTilAar)
{ } // Barnebok

//-------------------------------------------------------------------
void Barnebok::finnAldersgruppe(int &fraAarUt, int &tilAarUt) const
{
  fraAarUt = fraAar;
  tilAarUt = tilAar;
} // finnAldersgruppe

//-------------------------------------------------------------------
string Barnebok::finnBoktype() const
{
  return "Barnebok";
} // finnBoktype

//-------------------------------------------------------------------
string Barnebok::lagUtskrift() const
{
  ostringstream res;
  res << Bok::lagUtskrift() << ", aldersgruppe: " << fraAar <<
         " - " << tilAar << " r ";
  return res.str();
} // lagUtskrift

//-------------------------------------------------------------------
Skjoennlitteraer::Skjoennlitteraer(const string &initTittel,
                  const string *initForft, int initAntForft,
                  int initUtgAar, double initPris, int initAntSider,
                  const string &initIsbn, const string &initUtgLand,
                  sjanger initSjanger)
        : Bok(initTittel, initForft, initAntForft, initUtgAar, initPris, 
              initAntSider, initIsbn, initUtgLand),
          sjangeren(initSjanger)
{ } // Skjoennlitteraer

//-------------------------------------------------------------------
Skjoennlitteraer::sjanger Skjoennlitteraer::finnSjanger() const
{
  return sjangeren;
} // finnSjanger

//-------------------------------------------------------------------
string Skjoennlitteraer::lagUtskrift() const
{
  string tekst = "";
  switch (sjangeren) {
    case krim:
      tekst = "krim";
      break;
    case historiskRoman:
      tekst = "historisk roman";
      break;
    case samtidsroman:
      tekst = "samtidsroman";
      break;
    case dikt:
      tekst = "dikt";
      break;
    default:
      tekst = "";
      break;
    } // switch

  ostringstream res;
  res << Bok::lagUtskrift() << ", sjanger: " << tekst;
  return res.str();
} // lagUtskrift

//-------------------------------------------------------------------
string Skjoennlitteraer::finnBoktype() const
{
  return "Skjnnlitteratur";
} // finnBoktype

//-------------------------------------------------------------------
Fagbok::Fagbok(const string &initTittel,
               const string *initForft, int initAntForft,
               int initUtgAar, double initPris, int initAntSider,
               const string &initIsbn, const string &initUtgLand,
               emne initEmne)
        : Bok(initTittel, initForft, initAntForft, initUtgAar, initPris, 
              initAntSider, initIsbn, initUtgLand),
          emnet(initEmne)
{ } // Fagbok

//-------------------------------------------------------------------
Fagbok::emne Fagbok::finnEmne() const
{
  return emnet;
} // finnEmne

//-------------------------------------------------------------------
string Fagbok::finnBoktype() const
{
  return "Fagbok";
} // finnBoktype

//-------------------------------------------------------------------
string Fagbok::lagUtskrift() const
{
  string tekst = "";
  switch (emnet) {
    case historie:
      tekst = "historie";
      break;
    case matematikk:
      tekst = "matematikk";
      break;
    case data:
      tekst = "data";
      break;
    case samfunnsfag:
      tekst = "samfunnsfag";
      break;
    default:
      tekst = "";
      break;
    } // switch

  ostringstream res;
  res << Bok::lagUtskrift() << ", emne: " << tekst;
  return res.str();
} // lagUtskrift