//---------------------------------------------------------------------
//
// boeker.h
//
// Klasser som beskriver bker av ulik type.
//
//
#ifndef BOEKER_DEFINERT
  #define BOEKER_DEFINERT

  #include <string>

  using namespace std;

  class Bok {
    public:      
      virtual ~Bok();

      const string &finnTittel() const;
      const string *finnForfattere() const;
      int           finnAntForft() const;
      int           finnUtgAar() const;
      double        finnPris() const;
      int           finnAntSider() const;
      const string &finnIsbn() const;
      const string &finnUtgLand() const;

      virtual string finnBoktype() const = 0;
      virtual string lagUtskrift() const;

    protected:
      Bok(
        const string &initTittel,   // Inn
        const string *initForft,    // Inn
        int           initAntForft, // Inn
        int           initUtgAar,   // Inn
        double        initPris,     // Inn
        int           initAntSider, // Inn
        const string &initIsbn,     // Inn
        const string &initUtgLand); // Inn

      // Vi har peker som datamedlem, og m lage kopikonstruktr,
      // tilordningoperator og destruktr.
      Bok(
        const Bok& original);  // Inn
      Bok &operator=(
        const Bok& denAndre);  // Inn

    private:
      void kopierForftTabell(
             const string *initForft,     // Inn
             int           initAntForft); // Inn

      string  tittel;
      string *forfattere; // peker til tabell
      int     antForft;   // tabell-lengde
      int     utgAar;
      double  pris;
      int     antSider;
      string  isbn;
      string  utgLand;
  }; // Bok

  //-------------------------------------------------------------------
  class Barnebok: public Bok {
    public:
      Barnebok(
        const string  &initTittel,  // Inn
        const string *initForft,    // Inn
        int           initAntForft, // Inn
        int           initUtgAar,   // Inn
        double        initPris,     // Inn
        int           initAntSider, // Inn
        const string &initIsbn,     // Inn
        const string &initUtgLand,  // Inn
        int           initFraAar,   // Inn
        int           initTilAar);  // Inn

      void finnAldersgruppe(
             int &fraAarUt,         // Ut
             int &tilAarUt) const;  // Ut

      virtual string finnBoktype() const;
      virtual string lagUtskrift() const;

    private:
      int fraAar;
      int tilAar;
  };  // Barnebok

  //-------------------------------------------------------------------
  class Skjoennlitteraer: public Bok {
    public:
      enum sjanger {krim, historiskRoman, samtidsroman, dikt};

      Skjoennlitteraer(
        const string  &initTittel,   // Inn
        const string  *initForft,    // Inn
        int            initAntForft, // Inn
        int            initUtgAar,   // Inn
        double         initPris,     // Inn
        int            initAntSider, // Inn
        const string  &initIsbn,     // Inn
        const string  &initUtgLand,  // Inn
        sjanger        initSjanger); // Inn

      sjanger finnSjanger() const;

      virtual string finnBoktype() const;
      virtual string lagUtskrift() const;

    private:
      sjanger sjangeren;
  };  // Skjoennlitteraer

  //-------------------------------------------------------------------
  class Fagbok: public Bok {
    public:
      enum emne {historie, matematikk, data, samfunnsfag};

      Fagbok(
        const string  &initTittel,   // Inn
        const string  *initForft,    // Inn
        int            initAntForft, // Inn
        int            initUtgAar,   // Inn
        double         initPris,     // Inn
        int            initAntSider, // Inn
        const string  &initIsbn,     // Inn
        const string  &initUtgLand,  // Inn
        emne           initEmne);    // Inn    

      emne finnEmne() const;

      virtual string finnBoktype() const;
      virtual string lagUtskrift() const;

    private:
      emne emnet;
  };  // Fagbok
#endif
