//---------------------------------------------------------------------
//
// meny_publikasjoner.cpp
//
// Denne filen inneholder hoevdprogrammet med menyen.
// Programmet bruker funksjonen lesMenyValg() i filen nytte.h/nytte.cpp
//
// Lesing og skriving av data foregr p flgende mte:
// Hovedprogrammet nedenfor begynner med  lese inn hele filen og lagre
// dataene i et objekt av klassen PublSamling. Innlesingen ligger i
// funksjonen lesDataFraFil() som fins i denne cpp-filen.
// Etter behandling av dataene skrives de tilbake til datafilen i
// funksjonen skrivDataTilFil() som ogs fins her.
//
// Funksjonen lesDataFraFil() oppretter objekter av de ulike
// publikasjonsklassene. Frste linje for hver publikasjon p datafilen
// forteller hvilken datatype prublikasjonen er. P grunnlag av dette
// opprettes riktig type objekt, og dataene leses inn i konstruktren.
//
// Funksjonen skrivDataTilFil() bruker den virtuelle medlemsfunksjonen
// skrivTilFil() for  skrive ut data om et enkelt objekt.
//
// Menyen nedenfor gir oogs muligheten til  f skrevet ut informasjon
// om publikasjonene p skjermen. Samme funksjoner som for skriving til
// fil benyttes. Dette frer til at utskriftene til skjerm er relativt
// primitive (uten ledetekster).
//
// Det er ikke datakontroll ved innlesing fra tastaturet.
//

#include <iostream>
#include "nytte.h"
#include "samling_av_publ.h"

const string menytekst = "F: Nytt foredrag\n"
                         "S: Ny tidsskriftartikkel\n"
                         "R: Ny rapport\n"
                         "B: Ny bok\n"
                         "D: Ny dato p foredrag\n"
                         "T: Sk etter publikasjon, gitt tittel\n"
                         "1: Sk etter foredrag\n"
                         "2: Sk etter tidsskriftartikler\n"
                         "3: Sk etter rapporter\n"
                         "4: Sk etter bker\n"
                         "A: Avslutt.\n";
const string lovligValg = "FSRBDT1234A";
enum valg {nyttForedrag, nyTidsskriftArt, nyRapport, nyBok, nyDato, soekTittel,
           soekForedrag, soekTidsk, soekRapport, soekBok, avslutt};

void lesDataFraFil(const string &filnavn, PublSamling &samlingen);
void skrivDataTilFil(const string &filnavn, const PublSamling &samlingen);
void utfoerValg(valg valget, PublSamling &samlingen);
void soek(const string &type, const PublSamling &samlingen);

int main()
{
  const string filnavn = "publ_samling.txt";
  PublSamling samlingen;
  lesDataFraFil(filnavn, samlingen);
  valg valget = (valg) lesMenyvalg(menytekst, lovligValg);
  while (valget != avslutt) {
    utfoerValg(valget, samlingen);
    valget = (valg) lesMenyvalg(menytekst, lovligValg);
  }
  skrivDataTilFil(filnavn, samlingen);
  return 0;
} // main


//---------------------------------------------------------------------
//
// Leser hele datafilen
//
void lesDataFraFil(
       const string &filnavn,     // Inn
       PublSamling  &samlingen)   // Inn/ut
{
  ifstream innfil;
  innfil.open(filnavn.c_str());
  if (innfil) {  // fr til  pne filen
    Publikasjon *nyPubl = 0;
    string type = lesLinje(innfil);
    while (!innfil.eof()) {
      if (type == "Bok") nyPubl = new Bok(innfil);
      else if (type == "Foredrag") nyPubl = new Foredrag(innfil);
      else if (type == "Tidsskriftartikkel") nyPubl = new Tidsskriftartikkel(innfil);
      else if (type == "Rapport") nyPubl = new Rapport(innfil);
      else avsluttMedFeilmelding("Ukjent type. " + type + " Avslutter.");
      samlingen.registrerNyPublikasjon(nyPubl);
      type = lesLinje(innfil);
      } // while
    innfil.clear();
    innfil.close();
  }
} // lesDataFraFil

//---------------------------------------------------------------------
//
// Leser hele datafilen
//
void skrivDataTilFil(
       const string      &filnavn,       // Inn
       const PublSamling &samlingen)     // Inn
{
  ofstream utfil;
  utfil.open(filnavn.c_str());
  if (!utfil) avsluttMedFeilmelding("Kan ikke pne filen.");
  for (int i = 0; i < samlingen.finnAntallPublikasjoner(); i++) {
    const Publikasjon *publ = samlingen.finnPublikasjon(i);
    publ->skrivTilFil(utfil);
    }
  utfil.close();
} // skrivDataTilFil

//---------------------------------------------------------------------
//
// Vedlikeholder publikasjonssamlingen
//
void utfoerValg(
  valg         valget,        // Inn
  PublSamling &samlingen)     // Inn/ut
{
  string tittel;
  string *forft;
  int antForft;
  if (valget == nyttForedrag || valget == nyTidsskriftArt ||
      valget == nyRapport || valget == nyBok) {
    cout << "Tittel: ";
    tittel = lesLinje(cin);
    cout << "Antall forfattere: ";
    cin >> antForft;
    forft = new string[antForft];
    for (int i = 0; i < antForft; i++) {
      cout << "Forfatter nr " << (i + 1) << ": ";
      forft[i] = lesLinje(cin);
      }
    } // if ny publikasjon

  string sammendrag;
  if (valget == nyttForedrag || valget == nyTidsskriftArt ||
    valget == nyRapport) {
    cout << "Sammendrag, skriv \"xxx\" p siste linje:\n";
    string linje  = lesLinje(cin);
    while (linje != "xxx") {
      sammendrag += (linje + "\n");
      linje  = lesLinje(cin);
      }
    } // if ny publikasjon med sammendrag

  switch (valget) {
    case nyttForedrag: {
      Publikasjon *nyPubl = new Foredrag(tittel, forft, antForft, sammendrag);
      samlingen.registrerNyPublikasjon(nyPubl);
      cout << "Nytt foredrag registrert\n";
      }  // case
      break;

    case nyTidsskriftArt: {
      int nr;
      cout << "Navn p tidsskriftet: ";
      string navn = lesLinje(cin);
      cout << "Nr: ";
      cin >> nr;
      Publikasjon *nyPubl = new Tidsskriftartikkel(tittel, forft,
                                  antForft, sammendrag, navn, nr);
      samlingen.registrerNyPublikasjon(nyPubl);
      cout << "Ny tidsskriftartikkel registrert\n";
      }  // case
      break;

    case nyRapport: {
      cout << "Navn p firma: ";
      string firma = lesLinje(cin);
      cout << "Dato: ";
      string dato = lesLinje(cin);
      Publikasjon *nyPubl = new Rapport(tittel, forft,
                                  antForft, sammendrag, firma, dato);
      samlingen.registrerNyPublikasjon(nyPubl);
      cout << "Ny rapport registrert\n";
      }  // case
      break;

    case nyBok: {
      cout << "Navn p forlag: ";
      string forlag = lesLinje(cin);
      cout << "r: ";
      int aar;
      cin >> aar;
      cout << "Innhold, oppgi filnavn:\n";
      string innhold = lesLinje(cin);
      Publikasjon *nyPubl =
        new Bok(tittel, forft, antForft, forlag, aar, innhold);
      samlingen.registrerNyPublikasjon(nyPubl);
      cout << "Ny bok registrert\n";
      }  // case
      break;

    case nyDato: {
      cout << "Oppgi tittelen p foredraget: ";
      string tittel = lesLinje(cin);
      const Publikasjon *publ = samlingen.finnPublikasjon(tittel);
      if (publ != 0) {
        string type = publ->finnPublikasjonstype();
        if (type == "Foredrag") {
          cout << "Oppgi ny dato: ";
          string dato = lesLinje(cin);
          // behver ikke sjekke returverdi, vet at tittel ogtype er korrekt
          samlingen.registrerNyDatoForedrag(tittel, dato);
          cout << "Datoen er registrert\n";
          }
        else cout << "Publikasjonen er ikke et foredrag, men " << type << endl;
        }
      else cout << "Publikasjon med denne tittelen fins ikke\n";
      } // case
      break;

    case soekTittel: {
      cout << "Oppgi tittelen: ";
      string tittel = lesLinje(cin);
      const Publikasjon *publ = samlingen.finnPublikasjon(tittel);
      if (publ != 0) {
        cout << "Type: " << publ->finnPublikasjonstype() << endl;
        cout << "Rdata flger:\n";
        publ->skrivTilFil(cout);
        }
      else cout << "Publikasjon med denne tittelen fins ikke.\n";
      }
      break;

    case soekForedrag:
      soek("Foredrag", samlingen);
      break;

    case soekTidsk:
      soek("Tidsskriftartikkel", samlingen);
      break;

    case soekRapport:
      soek("Rapport", samlingen);
      break;

    case soekBok:
      soek("Bok", samlingen);
      break;

    default:
      break;
    } // switch
} // utfoerValg

//---------------------------------------------------------------------
//
// Sker etter publikasjoner av en bestemt type
//
void soek(
  const string      &type,         // Inn
  const PublSamling &samlingen)    // Inn
{
  for (int i = 0; i < samlingen.finnAntallPublikasjoner(); i++) {
    const Publikasjon *publ = samlingen.finnPublikasjon(i);
    if (publ->finnPublikasjonstype() == type) {
      publ->skrivTilFil(cout);
      cout << endl;
      }
    }
} // soek

