//---------------------------------------------------------------------
//
// publikasjoner.cpp - implementasjonsfil
//
// Klasser som beskriver faglige publikasjoner av ulik type.
//
//

#include "nytte.h"
#include "publikasjoner.h"

Publikasjon::Publikasjon(ifstream &innfil)
{  
  tittel = lesLinje(innfil);
  innfil >> antForft;
  forfattere = new string[antForft];
  for (int i = 0; i < antForft; i++) {
    forfattere[i] = lesLinje(innfil);
    }
} // Publikasjon

//---------------------------------------------------------------------
Publikasjon:: Publikasjon(const string &initTittel,
                 const string *initForft, int initAntForft)
            :  tittel(initTittel)
{
  forfattere = 0;
  antForft = 0;
  kopierForftTabell(initForft, initAntForft);
} // Publikasjon

//---------------------------------------------------------------------
Publikasjon::~Publikasjon()
{
  if (forfattere != 0) delete [] forfattere;
} // ~Publikasjon

//---------------------------------------------------------------------
void Publikasjon::kopierForftTabell(const string *initForft,
                                    int initAntForft)
{
  if (forfattere != 0) delete [] forfattere;
  antForft = initAntForft;
  forfattere = new string[antForft];
  for (int i = 0; i < antForft; i++) forfattere[i] = initForft[i];
} // kopierForftTabell

//---------------------------------------------------------------------
const string &Publikasjon::finnTittel() const
{
  return tittel;
} // finnTittel

//---------------------------------------------------------------------
const string *Publikasjon::finnForfattere() const
{
  return forfattere;
} // finnForfattere

//---------------------------------------------------------------------
int Publikasjon::finnAntForft() const
{
  return antForft;
} // finnAntForft

//---------------------------------------------------------------------
void Publikasjon::skrivTilFil(ostream &utfil) const
{
  utfil << finnPublikasjonstype() << endl;
  utfil << tittel << endl;
  utfil << antForft << endl;
  for (int i = 0; i < antForft; i++) utfil << forfattere[i] << endl;
} // skrivTilFil

//---------------------------------------------------------------------
Bok::Bok(ifstream &innfil) 
   : Publikasjon(innfil) 
{
  forlag = lesLinje(innfil);
  innfil >> utgAar;
  innhold = lesLinje(innfil);
} // Bok

//---------------------------------------------------------------------
Bok::Bok(const string &initTittel, const string *initForft,
         int initAntForft, const string &initForlag, int initUtgAar,
         const string &initInnhold)
   : Publikasjon(initTittel, initForft, initAntForft),
     forlag(initForlag),
     utgAar(initUtgAar),
     innhold(initInnhold)
{ };  // Bok

//-------------------------------------------------------------------
const string &Bok::finnForlag() const
{
  return forlag;
} // finnForlag

//-------------------------------------------------------------------
int Bok::finnUtgAar() const
{
  return utgAar;
} // finnUtgaar

//-------------------------------------------------------------------
const string &Bok::finnInnhold() const
{
  return innhold;
} // finnInnhold

//-------------------------------------------------------------------
string Bok::finnPublikasjonstype() const
{
  return "Bok";
} // finnPublikasjonstype

//-------------------------------------------------------------------
void Bok::skrivTilFil(ostream &utfil) const
{
  Publikasjon::skrivTilFil(utfil);
  utfil << forlag << endl;
  utfil << utgAar << endl;
  utfil << innhold << endl;
}  // skrivTilFil

//-------------------------------------------------------------------
PublMedSammendrag::PublMedSammendrag(ifstream &innfil)
                 : Publikasjon(innfil)
{
  string linje = lesLinje(innfil);
  while (linje != "xxx") {
    sammendrag += linje;
    sammendrag += "\n";
    linje = lesLinje(innfil);
    }  
} // PublMedSammendrag

//-------------------------------------------------------------------
PublMedSammendrag::PublMedSammendrag(const string &initTittel,
                   const string *initForft, int initAntForft,
                   const string &initSammendrag)
                 : Publikasjon(initTittel, initForft, initAntForft),
                   sammendrag(initSammendrag)
{ } // PublMedSammendrag

//-------------------------------------------------------------------
const string &PublMedSammendrag::finnSammendrag() const
{
  return sammendrag;
} // finnSammendrag

//-------------------------------------------------------------------
void PublMedSammendrag::skrivTilFil(ostream &utfil) const
{
  Publikasjon::skrivTilFil(utfil);
  utfil << sammendrag << endl << "xxx" << endl;
}  // skrivTilFil

//-------------------------------------------------------------------
Foredrag::Foredrag(ifstream &innfil)
        : PublMedSammendrag(innfil)
{
  int antall;
  innfil >> antall;
  for (int i = 0; i < antall; i++) {
    string dato;
    innfil >> dato;
    datoer.push_back(dato);
    }
} // Foredrag

//-------------------------------------------------------------------
Foredrag::Foredrag(const string &initTittel, const string *initForft,
                   int initAntForft, const string &initSammendrag)
                 : PublMedSammendrag(initTittel, initForft,
                     initAntForft, initSammendrag)
{ } // Foredrag

//-------------------------------------------------------------------
const vector<const string> Foredrag::finnDatoer() const
{
  return datoer;
} // finnDatoer

//-------------------------------------------------------------------
void Foredrag::registrerNyDato(const string &nyDato)
{
  datoer.push_back(nyDato);
} // registrerNyDato

//-------------------------------------------------------------------
string Foredrag::finnPublikasjonstype() const
{
  return "Foredrag";
} // finnPublikasjonstype

//-------------------------------------------------------------------
void Foredrag::skrivTilFil(ostream &utfil) const
{
  PublMedSammendrag::skrivTilFil(utfil);
  utfil << (int) datoer.size() << endl;
  for (int i = 0; i < (int) datoer.size(); i++) {
    utfil << datoer[i] << endl;
    }
}  // skrivTilFil

//-------------------------------------------------------------------
Tidsskriftartikkel::Tidsskriftartikkel(ifstream &innfil)
                  : PublMedSammendrag(innfil)
{
  navn = lesLinje(innfil);
  innfil >> nr;
} // Tidsskriftartikkel

//-------------------------------------------------------------------
Tidsskriftartikkel::Tidsskriftartikkel(const string &initTittel,
                      const string *initForft, int initAntForft,
                      const string &initSammendrag,
                      const string &initNavn, int initNr)
                 :  PublMedSammendrag(initTittel, initForft,
                     initAntForft, initSammendrag),
                    navn(initNavn),
                    nr(initNr)
{ } // Tidsskriftartikkel

//-------------------------------------------------------------------
const string &Tidsskriftartikkel::finnNavn() const
{
  return navn;
} // finnNavn

//-------------------------------------------------------------------
int Tidsskriftartikkel::finnNr() const
{
  return nr;
} // finnNr

//-------------------------------------------------------------------
string Tidsskriftartikkel::finnPublikasjonstype() const
{
  return "Tidsskriftartikkel";
} // finnPublikasjonstype

//-------------------------------------------------------------------
void Tidsskriftartikkel::skrivTilFil(ostream &utfil) const
{
  PublMedSammendrag::skrivTilFil(utfil);
  utfil << navn << endl;
  utfil << nr << endl;
}  // skrivTilFil

//-------------------------------------------------------------------
Rapport::Rapport(ifstream &innfil)
       : PublMedSammendrag(innfil)
{
 firma = lesLinje(innfil);
 dato = lesLinje(innfil);
} // Rapport

//-------------------------------------------------------------------
Rapport::Rapport(const string &initTittel,
           const string *initForft, int initAntForft,
           const string &initSammendrag,
           const string &initFirma, const string &initDato)
       : PublMedSammendrag(initTittel, initForft, initAntForft,
           initSammendrag),
           firma(initFirma),
           dato(initDato)
{ } // Rapport

//-------------------------------------------------------------------
const string &Rapport::finnFirma() const
{
  return firma;
} // finnFirma

//-------------------------------------------------------------------
const string &Rapport::finnDato() const
{
  return dato;
} // finnDato

//-------------------------------------------------------------------
string Rapport::finnPublikasjonstype() const
{
  return "Rapport";
} // finnPublikasjonstype

//-------------------------------------------------------------------
void Rapport::skrivTilFil(ostream &utfil) const
{
  PublMedSammendrag::skrivTilFil(utfil);
  utfil << firma << endl;
  utfil << dato << endl;
}  // skrivTilFil
