//---------------------------------------------------------------------
//
// publikasjoner.h
//
// Klasser som beskriver faglige publikasjoner av ulik type.
//
//
#ifndef PUBLIKASJONER_DEFINERT
  #define PUBLIKASJONER_DEFINERT

  #include <fstream>
  #include <iostream>
  #include <string>
  #include <vector>

  using namespace std;

  class Publikasjon {
    public:
      virtual ~Publikasjon();

      const string &finnTittel() const;
      const string *finnForfattere() const;
      int           finnAntForft() const;

      virtual string finnPublikasjonstype() const = 0;
      virtual void   skrivTilFil(
                       ostream &utfil) const; // Inn/ut

    protected:
      Publikasjon( // brukes nr data leses fra fil
        ifstream     &innfil);      // Inn/ut

      Publikasjon(
        const string &initTittel,   // Inn
        const string *initForft,    // Inn
        int           initAntForft);// Inn

      // Vi har peker som datamedlem, og m lage kopikonstruktr,
      // tilordningoperator og destruktr. Imidlertid vurderer
      // vi det slik at det ikke er behov for  kopiere
      // publikasjonsobjekter, derfor deklarerer vi kopikonstruktr
      // og tilordningsoperator privat, og unnlater  implementere disse.
      // Destruktr er deklarert foran.
      Publikasjon(const Publikasjon& original);
      Publikasjon &operator=(const Publikasjon& denAndre);

    private:
      void kopierForftTabell(
             const string *initForft,     // Inn
             int           initAntForft); // Inn

      string  tittel;
      string *forfattere; // peker til tabell
      int     antForft;   // tabell-lengde
  }; // Publikasjon

  //-------------------------------------------------------------------
  class Bok: public Publikasjon {
    public:
      Bok(
        ifstream     &innfil);        // Inn/ut

      Bok(
        const string &initTittel,     // Inn
        const string *initForft,      // Inn
        int           initAntForft,   // Inn
        const string &initForlag,     // Inn
        int           initUtgAar,     // Inn
        const string &initInnhold);   // Inn

      const string &finnForlag() const;
      int           finnUtgAar() const;
      const string &finnInnhold() const;

      virtual string finnPublikasjonstype() const;    
      virtual void   skrivTilFil(
                       ostream &utfil) const; // Inn/ut
    private:
      string forlag;
      int    utgAar;
      string innhold;  // navn p fil der innholdet ligger
  };  // Bok

  //-------------------------------------------------------------------
  class PublMedSammendrag: public Publikasjon {
    public:
      const string &finnSammendrag() const;

      virtual void   skrivTilFil(
                      ostream &utfil) const; // Inn/ut

    protected:
      PublMedSammendrag(
        ifstream     &innfil);        // Inn/ut

      PublMedSammendrag(
        const string  &initTittel,    // Inn
        const string *initForft,      // Inn
        int           initAntForft,   // Inn
        const string &initSammendrag);// Inn

    private:
      string sammendrag;
  };  // PublMedSammendrag

  //-------------------------------------------------------------------
  class Foredrag: public PublMedSammendrag {
    public:
      Foredrag(
        ifstream     &innfil);        // Inn/ut

      Foredrag(
        const string  &initTittel,    // Inn
        const string  *initForft,     // Inn
        int            initAntForft,  // Inn
        const string &initSammendrag);// Inn

      // Foredragsdatoene samles i en vektor som kan utvides
      const vector<const string> finnDatoer() const;
      void                       registrerNyDato(
                                   const string &nyDato);  // Inn

      virtual string finnPublikasjonstype() const;
      virtual void   skrivTilFil(
                       ostream &utfil) const; // Inn/ut

    private:
      vector<const string> datoer;
  };  // Foredrag

  //-------------------------------------------------------------------
  class Tidsskriftartikkel: public PublMedSammendrag {
    public:
      Tidsskriftartikkel(
        ifstream     &innfil);        // Inn/ut

      Tidsskriftartikkel(
        const string &initTittel,     // Inn
        const string *initForft,      // Inn
        int           initAntForft,   // Inn
        const string &initSammendrag, // Inn
        const string &initNavn,       // Inn
        int           initNr);        // Inn

      const string &finnNavn() const;
      int           finnNr() const;

      virtual string finnPublikasjonstype() const;
      virtual void   skrivTilFil(
                       ostream &utfil) const; // Inn/ut

    private:
      string navn;
      int    nr;
  };  // Tidsskriftartikkel

  //-------------------------------------------------------------------
  class Rapport: public PublMedSammendrag {
    public:
      Rapport(
        ifstream     &innfil);        // Inn/ut

      Rapport(
        const string &initTittel,     // Inn
        const string *initForft,      // Inn
        int           initAntForft,   // Inn
        const string &initSammendrag, // Inn
        const string &initFirma,      // Inn
        const string &initDato);      // Inn

      const string &finnFirma() const;
      const string &finnDato() const;

      virtual string finnPublikasjonstype() const;
      virtual void   skrivTilFil(
                       ostream &utfil) const; // Inn/ut

    private:
      string firma;
      string dato;
  };  // Rapport
#endif
