//-------------------------------------------------------------------
//
// samling_av_publ.h - en samling med publikasjoner
//

#ifndef PUBL_SAMLING_DEFINERT
  #define PUBL_SAMLING_DEFINERT

  #include <string>
  #include <vector>
  #include "publikasjoner.h"

  using namespace std;

  class PublSamling {
    public:
      PublSamling();
      ~PublSamling(); 

      //
      // Funksjonen legger peker til publikasjonsobjektet inn i vektoren.
      // Objektet m ligge i det frie lageret. Destruktren
      // srger for  slette objektene.
      //
      // Dersom publikasjon med samme tittel fins fra fr, registreres
      // ikke publikasjon. Da returnerer funksjonen false.
      //
      bool       registrerNyPublikasjon(
                   Publikasjon *&nyPubl);   // Inn/ut (pekeren settes lik 0)

      // Flgende to funksjoner kan brukes til  hente ut alle publikasjonene
      int        finnAntallPublikasjoner() const;
      const Publikasjon *finnPublikasjon(  // returnerer 0 hvis ugyldig indeks
                   int indeks) const;           // Inn

      const Publikasjon *finnPublikasjon(  // returnerer 0 hvis tittel ikke finnes
                          const string &tittel) const; // Inn
      
      // Funksjonen returnerer false hvis tittelen ikke fins, 
      // eller det ikke er et foredrag som har denne tittelen
      bool registrerNyDatoForedrag(  
                          const string &tittel,  // Inn
                          const string &nyDato); // Inn

    private:
      // Klassen inneholder en vektor av pekere med objektene i det frie lageret.
      // Det tilsier at kopikonstruktr, tilordningsoperator og destruktr
      // m programmeres - dersom det er behov for dem. Den siste
      // m vi lage, de to frste anser vi som undvendige.
      // Derfor deklarerer vi dem privat, da vil klienter ikke kunne
      // n dem.
      PublSamling(
        const PublSamling &original);  // Inn
      PublSamling &operator=(
        const PublSamling& denAndre);  // Inn

      vector<Publikasjon *> publikasjoner;
  }; // PublSamling
#endif